<?php
namespace Mm\Filapp\Controller;


/***
 *
 * This file is part of the "FirstInvestLocal" Extension for TYPO3 CMS.
 *
 * For the full copyright and license information, please read the
 * LICENSE.txt file that was distributed with this source code.
 *
 *  (c) 2020 Andreas Jud <ajud@millemedia.de>, Millemedia GmbH
 *
 ***/

use Mm\Filapp\Property\TypeConverter\UploadedFileReferenceConverter;
use TYPO3\CMS\Extbase\Property\PropertyMappingConfiguration;

/**
 * ProjectController
 */
class ProjectController extends \TYPO3\CMS\Extbase\Mvc\Controller\ActionController
{

    /**
     * projectRepository
     * 
     * @var \Mm\Filapp\Domain\Repository\ProjectRepository
     */
    protected $projectRepository = null;

    /**
     * branchRepository
	 *
     * @TYPO3\CMS\Extbase\Annotation\Inject
     * @var \Mm\Filapp\Domain\Repository\BranchRepository
     */
    protected $branchRepository = null;

    /**
     * companytypeRepository
	 *
     * @TYPO3\CMS\Extbase\Annotation\Inject
     * @var \Mm\Filapp\Domain\Repository\CompanytypeRepository
     */
    protected $companytypeRepository = null;

    /**
	 *
     * @TYPO3\CMS\Extbase\Annotation\Inject
     * @var \SJBR\StaticInfoTables\Domain\Repository\CountryRepository
     */
    protected $countryRepository = null;

    /**
	 *
     * @TYPO3\CMS\Extbase\Annotation\Inject
     * @var \SJBR\StaticInfoTables\Domain\Repository\CountryZoneRepository
     */
    protected $countryZoneRepository = null;

    /**
	 *
     * @TYPO3\CMS\Extbase\Annotation\Inject
     * @var \TYPO3\CMS\Extbase\Domain\Repository\FrontendUserRepository
     */
    protected $userRepository = null;


    /**
     * @param \Mm\Filapp\Domain\Repository\ProjectRepository $projectRepository
     */
    public function injectProjectRepository(\Mm\Filapp\Domain\Repository\ProjectRepository $projectRepository)
    {
        $this->projectRepository = $projectRepository;
    }

    /**
     * action list
     * 
     * @return void
     */
    public function listAction()
    {
        $projects = $this->projectRepository->findAll();
        $feuser = $this->userRepository->findByUid($GLOBALS['TSFE']->fe_user->user['uid']);
        $this->view->assign('feuser', $feuser);
        $this->view->assign('projects', $projects);
    }

    /**
     * action show
     * 
     * @param \Mm\Filapp\Domain\Model\Project $project
     * @return void
     */
    public function showAction(\Mm\Filapp\Domain\Model\Project $project)
    {
        $this->view->assign('project', $project);
    }

    /**
     * action new
     * 
     * @return void
     */
    public function newAction()
    {
        $years = [];
        $years['last'] = date('Y', strtotime(date("Y", time()) . " - 1 year"));
        $years['two'] = date('Y', strtotime(date("Y", time()) . " - 2 year"));
        $years['three'] = date('Y', strtotime(date("Y", time()) . " - 3 year"));
        $this->view->assign('years', $years);
        $branches = $this->branchRepository->findAll();
        $this->view->assign('branches', $branches);
        $cotypes = $this->companytypeRepository->findAll();
        $this->view->assign('cotypes', $cotypes);
        $feuser = $this->userRepository->findByUid($GLOBALS['TSFE']->fe_user->user['uid']);
        $this->view->assign('feuser', $feuser);
    }

    /**
     * Set TypeConverter option for image upload
     */
    public function initializeCreateAction()
    {
        $this->setTypeConverterConfigurationForImageUpload('newProject');
    }

    /**
     * action create
     * 
     * @param \Mm\Filapp\Domain\Model\Project $newProject
     * @return void
     */
    public function createAction(\Mm\Filapp\Domain\Model\Project $newProject)
    {
        $this->addFlashMessage('The object was created. Please be aware that this action is publicly accessible unless you implement an access check. See https://docs.typo3.org/typo3cms/extensions/extension_builder/User/Index.html', '', \TYPO3\CMS\Core\Messaging\AbstractMessage::WARNING);
        $this->projectRepository->add($newProject);
        $this->redirect('list');
    }

    /**
     * Set TypeConverter option for image upload
     */
    public function initializeUpdateAction()
    {
        $this->setTypeConverterConfigurationForImageUpload('project');
    }

    /**
     * action edit
     * 
     * @param \Mm\Filapp\Domain\Model\Project $project
     * @TYPO3\CMS\Extbase\Annotation\IgnoreValidation("project")
     * @return void
     */
    public function editAction(\Mm\Filapp\Domain\Model\Project $project)
    {
        $branches = $this->branchRepository->findAll();
        $this->view->assign('branches', $branches);
        $cotypes = $this->companytypeRepository->findAll();
        $this->view->assign('cotypes', $cotypes);
        $this->view->assign('project', $project);
    }

    /**
     * action update
     * 
     * @param \Mm\Filapp\Domain\Model\Project $project
     * @return void
     */
    public function updateAction(\Mm\Filapp\Domain\Model\Project $project)
    {
        $this->addFlashMessage('The object was updated. Please be aware that this action is publicly accessible unless you implement an access check. See https://docs.typo3.org/typo3cms/extensions/extension_builder/User/Index.html', '', \TYPO3\CMS\Core\Messaging\AbstractMessage::WARNING);
        $this->projectRepository->update($project);
        $this->redirect('list');
    }

    // Here we convert the property mapping using the property mapper
    public function initializeAction()
    {
        if ($this->arguments->hasArgument('newProject')) {
            $this->arguments->getArgument('newProject')->getPropertyMappingConfiguration()->forProperty('lastYear')->setTypeConverterOption('TYPO3\\CMS\\Extbase\\Property\\TypeConverter\\DateTimeConverter', \TYPO3\CMS\Extbase\Property\TypeConverter\DateTimeConverter::CONFIGURATION_DATE_FORMAT, 'Y');
            $this->arguments->getArgument('newProject')->getPropertyMappingConfiguration()->forProperty('year2')->setTypeConverterOption('TYPO3\\CMS\\Extbase\\Property\\TypeConverter\\DateTimeConverter', \TYPO3\CMS\Extbase\Property\TypeConverter\DateTimeConverter::CONFIGURATION_DATE_FORMAT, 'Y');
            $this->arguments->getArgument('newProject')->getPropertyMappingConfiguration()->forProperty('year3')->setTypeConverterOption('TYPO3\\CMS\\Extbase\\Property\\TypeConverter\\DateTimeConverter', \TYPO3\CMS\Extbase\Property\TypeConverter\DateTimeConverter::CONFIGURATION_DATE_FORMAT, 'Y');
        }
        if ($this->arguments->hasArgument('project')) {
            $this->arguments->getArgument('project')->getPropertyMappingConfiguration()->forProperty('lastYear')->setTypeConverterOption('TYPO3\\CMS\\Extbase\\Property\\TypeConverter\\DateTimeConverter', \TYPO3\CMS\Extbase\Property\TypeConverter\DateTimeConverter::CONFIGURATION_DATE_FORMAT, 'Y');
            $this->arguments->getArgument('project')->getPropertyMappingConfiguration()->forProperty('year2')->setTypeConverterOption('TYPO3\\CMS\\Extbase\\Property\\TypeConverter\\DateTimeConverter', \TYPO3\CMS\Extbase\Property\TypeConverter\DateTimeConverter::CONFIGURATION_DATE_FORMAT, 'Y');
            $this->arguments->getArgument('project')->getPropertyMappingConfiguration()->forProperty('year3')->setTypeConverterOption('TYPO3\\CMS\\Extbase\\Property\\TypeConverter\\DateTimeConverter', \TYPO3\CMS\Extbase\Property\TypeConverter\DateTimeConverter::CONFIGURATION_DATE_FORMAT, 'Y');
        }
        if ($this->arguments->hasArgument('update')) {
            $this->arguments->getArgument('update')->getPropertyMappingConfiguration()->forProperty('lastYear')->setTypeConverterOption('TYPO3\\CMS\\Extbase\\Property\\TypeConverter\\DateTimeConverter', \TYPO3\CMS\Extbase\Property\TypeConverter\DateTimeConverter::CONFIGURATION_DATE_FORMAT, 'Y');
            $this->arguments->getArgument('update')->getPropertyMappingConfiguration()->forProperty('year2')->setTypeConverterOption('TYPO3\\CMS\\Extbase\\Property\\TypeConverter\\DateTimeConverter', \TYPO3\CMS\Extbase\Property\TypeConverter\DateTimeConverter::CONFIGURATION_DATE_FORMAT, 'Y');
            $this->arguments->getArgument('update')->getPropertyMappingConfiguration()->forProperty('year3')->setTypeConverterOption('TYPO3\\CMS\\Extbase\\Property\\TypeConverter\\DateTimeConverter', \TYPO3\CMS\Extbase\Property\TypeConverter\DateTimeConverter::CONFIGURATION_DATE_FORMAT, 'Y');
        }
    }

    /**
     * action listOpen
     * 
     * @return void
     */
    public function listOpenAction()
    {
    }

    /**
     * action showOpen
     * 
     * @return void
     */
    public function showOpenAction()
    {
    }
    
    /**
     *
     */
    protected function setTypeConverterConfigurationForImageUpload($argumentName)
    {
        \TYPO3\CMS\Core\Utility\GeneralUtility::makeInstance(\TYPO3\CMS\Extbase\Object\Container\Container::class)
            ->registerImplementation(
                \TYPO3\CMS\Extbase\Domain\Model\FileReference::class,
                \Mn\Filapp\Domain\Model\FileReference::class
            );

        $uploadConfiguration = [
            UploadedFileReferenceConverter::CONFIGURATION_ALLOWED_FILE_EXTENSIONS => $GLOBALS['TYPO3_CONF_VARS']['GFX']['imagefile_ext'],
            UploadedFileReferenceConverter::CONFIGURATION_UPLOAD_FOLDER => '1:/content/',
        ];
        /** @var PropertyMappingConfiguration $newProjectConfiguration */
        $newExampleConfiguration = $this->arguments[$argumentName]->getPropertyMappingConfiguration();
        $newExampleConfiguration->forProperty('image')
            ->setTypeConverterOptions(
                UploadedFileReferenceConverter::class,
                $uploadConfiguration
            );
        $newExampleConfiguration->forProperty('imageCollection.0')
            ->setTypeConverterOptions(
                UploadedFileReferenceConverter::class,
                $uploadConfiguration
            );
    }

    
}
