<?php
namespace Mm\Filapp\Tests\Unit\Domain\Model;

/**
 * Test case.
 *
 * @author Andreas Jud <ajud@millemedia.de>
 */
class FeuserTest extends \TYPO3\TestingFramework\Core\Unit\UnitTestCase
{
    /**
     * @var \Mm\Filapp\Domain\Model\Feuser
     */
    protected $subject = null;

    protected function setUp()
    {
        parent::setUp();
        $this->subject = new \Mm\Filapp\Domain\Model\Feuser();
    }

    protected function tearDown()
    {
        parent::tearDown();
    }

    /**
     * @test
     */
    public function getFilTypeReturnsInitialValueForInt()
    {
        self::assertSame(
            0,
            $this->subject->getFilType()
        );
    }

    /**
     * @test
     */
    public function setFilTypeForIntSetsFilType()
    {
        $this->subject->setFilType(12);

        self::assertAttributeEquals(
            12,
            'filType',
            $this->subject
        );
    }

    /**
     * @test
     */
    public function getProjectReturnsInitialValueForProject()
    {
        $newObjectStorage = new \TYPO3\CMS\Extbase\Persistence\ObjectStorage();
        self::assertEquals(
            $newObjectStorage,
            $this->subject->getProject()
        );
    }

    /**
     * @test
     */
    public function setProjectForObjectStorageContainingProjectSetsProject()
    {
        $project = new \Mm\Filapp\Domain\Model\Project();
        $objectStorageHoldingExactlyOneProject = new \TYPO3\CMS\Extbase\Persistence\ObjectStorage();
        $objectStorageHoldingExactlyOneProject->attach($project);
        $this->subject->setProject($objectStorageHoldingExactlyOneProject);

        self::assertAttributeEquals(
            $objectStorageHoldingExactlyOneProject,
            'project',
            $this->subject
        );
    }

    /**
     * @test
     */
    public function addProjectToObjectStorageHoldingProject()
    {
        $project = new \Mm\Filapp\Domain\Model\Project();
        $projectObjectStorageMock = $this->getMockBuilder(\TYPO3\CMS\Extbase\Persistence\ObjectStorage::class)
            ->setMethods(['attach'])
            ->disableOriginalConstructor()
            ->getMock();

        $projectObjectStorageMock->expects(self::once())->method('attach')->with(self::equalTo($project));
        $this->inject($this->subject, 'project', $projectObjectStorageMock);

        $this->subject->addProject($project);
    }

    /**
     * @test
     */
    public function removeProjectFromObjectStorageHoldingProject()
    {
        $project = new \Mm\Filapp\Domain\Model\Project();
        $projectObjectStorageMock = $this->getMockBuilder(\TYPO3\CMS\Extbase\Persistence\ObjectStorage::class)
            ->setMethods(['detach'])
            ->disableOriginalConstructor()
            ->getMock();

        $projectObjectStorageMock->expects(self::once())->method('detach')->with(self::equalTo($project));
        $this->inject($this->subject, 'project', $projectObjectStorageMock);

        $this->subject->removeProject($project);
    }
}
